/*******************************************************\
* modlisation d'une surface d'eau                      *
* DrawTools.cpp                                         *
* ------------------------------------------------------*
* Georges AZZI / Georges CAMY                           * 
* Julie POULETTY / Arnaud QUENUM                        *
* IMAC Ing3 2002/2003 - Universit de Marne La Valle  *
* Copyright 2003                                        *
\*******************************************************/
#include "DrawTools.h"



/*******************************************************\
* Mthode:		drawCylinder                            *
* Description:  cre un cylindre avec coordonnes de    *
*               texture                                 *
\*******************************************************/
void DrawTools::drawCylinder(float r, float h,int dx, int dy,bool plein,double amax){
  int ix,iy;
  double a,pasa =amax/(double)dx,pash=h/(double)dy,x,y;

  for (iy=0; iy<dy; iy++){
     glBegin(GL_QUAD_STRIP);
     for (ix=0,a=0.0; ix<=dx; ix++,a+=pasa){
        x = r*cos(a);
	y = r*sin(a);
	glNormal3f(x,y,1.0f);
        glTexCoord2d(a/amax,(double)(iy+1)/dy);
	glVertex3d(x,y,(double)(iy+1)*pash);
	glTexCoord2d(a/amax,(double)(iy)/dy);
	glVertex3d(x,y,(double)iy*pash);
     }
     glEnd();
  }

  if (plein){
     glNormal3f(0.0f,0.0f,-1.0f);
     glBegin(GL_TRIANGLE_FAN);
     glTexCoord2d(0.0,0.0);
     glVertex3f(0.0,0.0,0.0);
     for (ix=0,a=amax; ix<=dx; ix++,a-=pasa){
        x = r*cos(a);
	y = r*sin(a);
	glTexCoord2d(x,y);
	glVertex3d(x,y,0.0);
     }
     glEnd();
     glNormal3f(0.0f,0.0f,1.0f);
     glBegin(GL_TRIANGLE_FAN);
     glTexCoord2d(0.0,0.0);
     glVertex3f(0.0,0.0,h);
     for (ix=0,a=0.0; ix<=dx; ix++,a+=pasa){
       x = r*cos(a);
       y = r*sin(a);
       glTexCoord2d(x,y);
       glVertex3d(x,y,h);
     }
     glEnd();
  }
}



/*******************************************************\
* Mthode:		drawCylinder                            *
* Description:  cre un cylindre complet avec           *
*               coordonnes de texture                  *
\*******************************************************/
void DrawTools::drawCylinder(float r, float h,int dx, int dy,bool plein){
  double amax=2.0*M_PI;
  int ix,iy;
  double a,pasa =amax/(double)dx,pash=h/(double)dy,x,y;

  for (iy=0; iy<dy; iy++){
     glBegin(GL_QUAD_STRIP);
     for (ix=0,a=0.0; ix<=dx; ix++,a+=pasa){
        x = r*cos(a);
	y = r*sin(a);
	glNormal3f(x,y,0.0f);
        glTexCoord2d(a/amax,(double)(iy+1)/dy);
	glVertex3d(x,y,(double)(iy+1)*pash);
	glTexCoord2d(a/amax,(double)(iy)/dy);
	glVertex3d(x,y,(double)iy*pash);
     }
     glEnd();
  }

  if (plein){
     glNormal3f(0.0f,0.0f,-1.0f);
     glBegin(GL_TRIANGLE_FAN);
     glTexCoord2d(0.0,0.0);
     glVertex3f(0.0,0.0,0.0);
     for (ix=0,a=amax; ix<=dx; ix++,a-=pasa){
        x = r*cos(a);
	y = r*sin(a);
	glTexCoord2d(x,y);
	glVertex3d(x,y,0.0);
     }
     glEnd();
     glNormal3f(0.0f,0.0f,1.0f);
     glBegin(GL_TRIANGLE_FAN);
     glTexCoord2d(0.0,0.0);
     glVertex3f(0.0,0.0,h);
     for (ix=0,a=0.0; ix<=dx; ix++,a+=pasa){
       x = r*cos(a);
       y = r*sin(a);
       glTexCoord2d(x,y);
       glVertex3d(x,y,h);
     }
     glEnd();
  }
}


/*******************************************************\
* Mthode:		drawInvertedCylinder                    *
* Description:  cre un cylindre (normale oppose) avec *
*               coordonnes de texture                  *
\*******************************************************/
void DrawTools::drawInvertedCylinder(float r, float h,int dx, int dy,bool plein,double amax){
  int ix,iy;
  double a,pasa =amax/(double)dx,pash=h/(double)dy,x,y;

  for (iy=0; iy<dy; iy++){
     glBegin(GL_QUAD_STRIP);
     for (ix=0,a=0.0; ix<=dx; ix++,a+=pasa){
        x = r*cos(a);
	y = r*sin(a);
	glNormal3f(x,y,-1.0f);
        glTexCoord2d(a/amax,(double)(iy+1)/dy);
	glVertex3d(x,y,(double)(iy+1)*pash);
	glTexCoord2d(a/amax,(double)(iy)/dy);
	glVertex3d(x,y,(double)iy*pash);
     }
     glEnd();
  }

  if (plein){
     glNormal3f(0.0f,0.0f,-1.0f);
     glBegin(GL_TRIANGLE_FAN);
     glTexCoord2d(0.0,0.0);
     glVertex3f(0.0,0.0,0.0);
     for (ix=0,a=amax; ix<=dx; ix++,a-=pasa){
        x = r*cos(a);
	y = r*sin(a);
	glTexCoord2d(x,y);
	glVertex3d(x,y,0.0);
     }
     glEnd();
     glNormal3f(0.0f,0.0f,-1.0f);
     glBegin(GL_TRIANGLE_FAN);
     glTexCoord2d(0.0,0.0);
     glVertex3f(0.0,0.0,h);
     for (ix=0,a=0.0; ix<=dx; ix++,a+=pasa){
       x = r*cos(a);
       y = r*sin(a);
       glTexCoord2d(x,y);
       glVertex3d(x,y,h);
     }
     glEnd();
  }
}


/*******************************************************\
* Mthode:		drawText                                *
* Description:  crit un texte  l'cran.				*
*               Taille de la police de 1 (petit)       * 
*				3 (grand)                               *
\*******************************************************/
void DrawTools::drawText(char * chaine, int size){
  switch (size){
     case 2:
        while(*chaine != '\0'){
           glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, *chaine);
           chaine++;
        }
        break;
     case 3:
        while(*chaine != '\0'){
           glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, *chaine);
           chaine++;
        }
        break;
     default:
       while(*chaine != '\0'){
           glutBitmapCharacter(GLUT_BITMAP_HELVETICA_10, *chaine);
           chaine++;
        }
        break;
  }
}



/*******************************************************\
* Mthode:		loadPPM                                 *
* Description:  charge une texture                      *
\*******************************************************/
unsigned char * DrawTools::loadPPM(char *titre,int *dx,int *dy){
  FILE *fichier;
  char chaine[255];
  unsigned char *image=NULL;

  if (!(fichier = fopen(titre, "rt"))){ 
    printf("fichier %s introuvable\n",titre);
    exit(0);
    return NULL;
  }
  fscanf(fichier,"%s\n",chaine);
  //printf("%s\n",chaine);
  do{ 
    fgets(chaine,255,fichier);
  } while (chaine[0]=='#');
  sscanf(chaine,"%d %d",dx,dy);
  //printf("%d,%d\n",*dx,*dy);
  fscanf(fichier,"%s\n",chaine);
  image = (unsigned char *) malloc (*dx**dy*3*sizeof(unsigned char));
  if (image==NULL){ 
    printf("erreur d'allocation de l'image de texture\n");
    return NULL;
  }
  fread (image, sizeof (unsigned char), *dx**dy * 3, fichier);
  fclose(fichier);

  return image;
}