/*******************************************************\
* modlisation d'une surface d'eau                      *
* WaterSurface.cpp                                      *
* ------------------------------------------------------*
* Georges AZZI / Georges CAMY                           * 
* Julie POULETTY / Arnaud QUENUM                        *
* IMAC Ing3 2002/2003 - Universit de Marne La Valle  *
* Copyright 2003                                        *
\*******************************************************/
#include "WaterSurface.h"

#ifndef DrawTools_h
#include "DrawTools.h"
#endif



/*******************************************************\
*                    constructeur                       *
\*******************************************************/
WaterSurface::WaterSurface(GLint newLength, GLint newWidth){
	// mettre les valeurs
	length = newLength;
	width = newWidth;
	
	// texture active
	texture = false;

	// charger la texture
   	image = DrawTools::loadPPM("water2.ppm",&dx,&dy);
    // demande d'un identificateur de texture libre
    glGenTextures(1,&id_texture);
	// choix de la texture courrante
	glBindTexture(GL_TEXTURE_2D,id_texture);
	// organisation des plans dans le tableau
	glPixelStorei(GL_UNPACK_ALIGNMENT,1);
	// paramtre de rptition de la texture
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	// la texture sera multiplie par la couleur du modle d'illumination
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	// construction d'une texture multichelle  partir d'une image
	gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB,dx,dy,GL_RGB,GL_UNSIGNED_BYTE,image);

	// crer la grille
	for (int i=0; i<length; i++){
		for (int j=0; j<width; j++){
			surface[i][j].setPosition(i,j,0);	
		}
		glEnd();
	}

	// choisir le modle par dfaut:  4 ressorts
	model = 1;

	// dfinir la raideur k par dfaut du ressort
	k = 0.5; // pour modle 4 ressorts 
	//k = 0.2; // pour modle 8 ressorts

	// perte d'energie
	friction = 0.999;
   
	// bouger trois points au milieu (pour dmonstration)
	GLVector force(0,0,0.5);
	applyForce(force,15,20);
	applyForce(force,16,20);
	applyForce(force,17,20);

	// dfinir le temps coul (utilis dans update)
	time = glutGet(GLUT_ELAPSED_TIME);
}



/*******************************************************\
* Mthode:		setK									*
* Description:  dfinit la raideur des ressorts			*
\*******************************************************/
void WaterSurface::setK(GLdouble x){
	k = x;
}



/*******************************************************\
* Mthode:		getFriction								*
* Description:  fournit la perte d'energie par friction	*
\*******************************************************/
GLdouble WaterSurface::getFriction(){
	return friction;
}



/*******************************************************\
* Mthode:		setFriction								*
* Description:  fournit la perte d'energie par friction	*
\*******************************************************/
void WaterSurface::setFriction(GLdouble x){
	friction = x;
}



/*******************************************************\
* Mthode:		getK									*
* Description:  fournit la raideur des ressorts			*
\*******************************************************/
GLdouble WaterSurface::getK(){
	return k;
}



/*******************************************************\
* Mthode:	reset					*
* Description:  reinitialise la surface			*
\*******************************************************/
void WaterSurface::reset(){
	GLVector v(0,0,0);
	for (int i=0; i<length; i++){
		for (int j=0; j<width; j++){
			surface[i][j].setPosition(i,j,0);
			surface[i][j].setSpeed(v);	
		}
		glEnd();
	}
	// blubb...
	GLVector force(0,0,0.2);
	applyForce(force,15,20);
	applyForce(force,16,20);
	applyForce(force,17,20);
}



/*******************************************************\
* Mthode:		setModel								*
* Description:  choisir le modle de calcul de la vague	*
*				1:  4 ressorts							*
*				2:  8 ressorts							*
*				3:  4 ressorts que mouvement vertical	*
\*******************************************************/
void WaterSurface::setModel(GLint number){
	model = number;
}



/*******************************************************\
* Mthode:		applyForce								*
* Description:  appliquer une force sur un lment de	*
*				la grille								*
\*******************************************************/
void WaterSurface::applyForce(GLVector force, GLint i, GLint j){
	surface[i][j].addForce(force);
}



/*******************************************************\
* Mthode:		update4SpringModel						*
* Description:  tablit le prochain tat de la surface	*
*				selon le modle  4 ressorts			*
\*******************************************************/
void WaterSurface::update4SpringModel(){
	GLVector f;
	GLdouble timestep;

	// dimensionner en fonction du temps coul (en secondes)
	timestep = (glutGet(GLUT_ELAPSED_TIME) - time) / 1000;

	// pour chaque point calculer le dsastre
	for (int i=1;i<40-1;i++){
		for (int j=1;j<29;j++){
			// calculer les forces appliqus  ce point en considrant la position0
			// et l'tirement, cd la distance euclidienne entre deux points
			
			/* en haut */
			f = surface[i][j].distanceVector(surface[i][j-1]);
			f = f - f.normalize();
			// calculer la force du ressort f = -kx
			f = f * (k);	
			// dimensionner f en fonction du temps coul
			//f = f * timestep;
			// appliquer la force
			surface[i][j].addForce(f); 

			/* en bas */
			f = surface[i][j].distanceVector(surface[i][j+1]);
			f = f - f.normalize();
			// calculer la force du ressort f = -kx
			f = f * (k);	
			// dimensionner f en fonction du temps coul
			//f = f * timestep;
			// appliquer la force
			surface[i][j].addForce(f); 

			/*  gauche */
			f = surface[i][j].distanceVector(surface[i-1][j]);
			f = f - f.normalize();
			// calculer la force du ressort f = -kx
			f = f * (k);
			// dimensionner f en fonction du temps coul
			//f = f * timestep;
			// appliquer la force
			surface[i][j].addForce(f); 

			/*  droite */
			f = surface[i][j].distanceVector(surface[i+1][j]);
			f = f - f.normalize();
			// calculer la force du ressort f = -kx
			f = f * (k);	
			// dimensionner f en fonction du temps coul
			//f = f * timestep;
			// appliquer la force
			surface[i][j].addForce(f);
			
			/* perte d'energie*/
			surface[i][j].setSpeed(surface[i][j].getSpeed()*friction); 
		}
	}

	// sauvegarder le temps
	time = glutGet(GLUT_ELAPSED_TIME);
}



/*******************************************************\
* Mthode:		update8SpringModel						*
* Description:  tablit le prochain tat de la surface	*
*				selon le modle  4 ressorts			*
\*******************************************************/
void WaterSurface::update8SpringModel(){
	GLVector f;
	GLdouble timestep;

	// dimensionner en fonction du temps coul (en secondes)
	timestep = (glutGet(GLUT_ELAPSED_TIME) - time) / 1000;

	// pour chaque point calculer le dsastre
	for (int i=1;i<40-1;i++){
		for (int j=1;j<29;j++){
			// calculer les forces appliqus  ce point en considrant la position0
			// et l'tirement, cd la distance euclidienne entre deux points
			
			/* en haut */
			f = surface[i][j].distanceVector(surface[i][j-1]);
			f = f - f.normalize();
			// calculer la force du ressort f = -kx
			f = f * (k);
			// dimensionner f en fonction du temps coul
			//f = f * timestep;
			// appliquer la force
			surface[i][j].addForce(f); 

			/* en bas */
			f = surface[i][j].distanceVector(surface[i][j+1]);
			f = f - f.normalize();
			// calculer la force du ressort f = -kx
			f = f * (k);		
			// dimensionner f en fonction du temps coul
			//f = f * timestep;
			// appliquer la force
			surface[i][j].addForce(f); 

			/*  gauche */
			f = surface[i][j].distanceVector(surface[i-1][j]);
			f = f - f.normalize();
			// calculer la force du ressort f = -kx
			f = f * (k);		
			// dimensionner f en fonction du temps coul
			//f = f * timestep;
			// appliquer la force
			surface[i][j].addForce(f); 

			/*  droite */
			f = surface[i][j].distanceVector(surface[i+1][j]);
			f = f - f.normalize();
			// calculer la force du ressort f = -kx
			f = f * (k);		
			// dimensionner f en fonction du temps coul
			//f = f * timestep;
			// appliquer la force
			surface[i][j].addForce(f); 


			/* haut droite */
			f = surface[i][j].distanceVector(surface[i+1][j-1]);
			f = f - (f.normalize() * 1.4142135);
			// calculer la force du ressort f = -kx
			f = f * (k);		
			// dimensionner f en fonction du temps coul
			//f = f * timestep;
			// appliquer la force
			surface[i][j].addForce(f); 

			/* haut gauche  */
			f = surface[i][j].distanceVector(surface[i-1][j-1]);
			f = f - (f.normalize() * 1.4142135);
			// calculer la force du ressort f = -kx
			f = f * (k);		
			// dimensionner f en fonction du temps coul
			//f = f * timestep;
			// appliquer la force
			surface[i][j].addForce(f); 

			/* bas gauche */
			f = surface[i][j].distanceVector(surface[i-1][j+1]);
			f = f - (f.normalize() * 1.4142135);
			// calculer la force du ressort f = -kx
			f = f * (k);		
			// dimensionner f en fonction du temps coul
			//f = f * timestep;
			// appliquer la force
			surface[i][j].addForce(f); 

			/* bas droite */
			f = surface[i][j].distanceVector(surface[i+1][j+1]);
			f = f - (f.normalize() * 1.4142135);
			// calculer la force du ressort f = -kx
			f = f * (k);		
			// dimensionner f en fonction du temps coul
			//f = f * timestep;
			// appliquer la force
			surface[i][j].addForce(f); 
			
			/* perte d'energie*/
			surface[i][j].setSpeed(surface[i][j].getSpeed()*friction); 
		}
	}

	// sauvegarder le temps
	time = glutGet(GLUT_ELAPSED_TIME);
}



/*******************************************************\
* Mthode:		update4SpringVerticalModel						*
* Description:  tablit le prochain tat de la surface	*
*				selon le modle  4 ressorts			*
\*******************************************************/
void WaterSurface::update4SpringVerticalModel(){
	GLVector f;
	GLdouble timestep;

	// dimensionner en fonction du temps coul (en secondes)
	timestep = (glutGet(GLUT_ELAPSED_TIME) - time) / 1000;

	// pour chaque point calculer le dsastre
	for (int i=1;i<40-1;i++){
		for (int j=1;j<29;j++){
			// calculer les forces appliqus  ce point en considrant la position0
			// et l'tirement, cd la distance euclidienne entre deux points
			
			/* en haut */
			f = surface[i][j].distanceVector(surface[i][j-1]);
			f = f - f.normalize();
			// calculer la force du ressort f = -kx
			f = f * (k);	
			f.setX(0);
			f.setY(0);
			// dimensionner f en fonction du temps coul
			//f = f * timestep;
			// appliquer la force
			surface[i][j].addForce(f); 

			/* en bas */
			f = surface[i][j].distanceVector(surface[i][j+1]);
			f = f - f.normalize();
			// calculer la force du ressort f = -kx
			f = f * (k);
			f.setX(0);
			f.setY(0);
			// dimensionner f en fonction du temps coul
			//f = f * timestep;
			// appliquer la force
			surface[i][j].addForce(f); 

			/*  gauche */
			f = surface[i][j].distanceVector(surface[i-1][j]);
			f = f - f.normalize();
			// calculer la force du ressort f = -kx
			f = f * (k);		
			f.setX(0);
			f.setY(0);
			// dimensionner f en fonction du temps coul
			//f = f * timestep;
			// appliquer la force
			surface[i][j].addForce(f); 

			/*  droite */
			f = surface[i][j].distanceVector(surface[i+1][j]);
			f = f - f.normalize();
			// calculer la force du ressort f = -kx
			f = f * (k);	
			f.setX(0);
			f.setY(0);
			// dimensionner f en fonction du temps coul
			//f = f * timestep;
			// appliquer la force
			surface[i][j].addForce(f); 
			
			/* perte d'energie*/
			surface[i][j].setSpeed(surface[i][j].getSpeed()*friction); 
		}
	}

	// sauvegarder le temps
	time = glutGet(GLUT_ELAPSED_TIME);
}



/*******************************************************\
* Mthode:		dessiner								*
* Description:  dessine la surface d'eau                *
\*******************************************************/
void WaterSurface::dessiner(){


	glPushMatrix();

	switch (model){
		case 1: update4SpringVerticalModel();
		break;
		case 2: update8SpringModel();
		break;
		case 3: update4SpringVerticalModel();
		break;
	}

	if (texture){
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
		glEnable(GL_TEXTURE_2D);
		glEnable(GL_LIGHTING);
		for (int i=1;i<40-2;i++){
			glBegin(GL_QUAD_STRIP);
			for (int j=1;j<29;j++){
				// update des point vers les nouvelles coordonnes
				surface[i][j].update();
				surface[i+1][j].update();
				// dessiner les points
				surface[i][j].dessiner();
				glTexCoord2d(0,0);
				surface[i+1][j].dessiner();
				glTexCoord2d(1,1);
			}
			glEnd();
		}
	}
	else{
		glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
		glDisable(GL_TEXTURE_2D);
		glDisable(GL_LIGHTING);
		glColor3f(0.0,0.65,0.8);
		for (int i=1;i<40-2;i++){
			glBegin(GL_QUAD_STRIP);
			for (int j=1;j<29;j++){
				// update des point vers les nouvelles coordonnes
				
				surface[i][j].update();
				surface[i+1][j].update();
				// dessiner les points
				surface[i][j].dessiner();
				surface[i+1][j].dessiner();
			}
			glEnd();
		}
	}

	glPopMatrix();
}
