/*******************************************************\
* modlisation d'une surface d'eau                      *
* WaterSurface2.cpp                                     *
* ------------------------------------------------------*
* Georges AZZI / Georges CAMY                           * 
* Julie POULETTY / Arnaud QUENUM                        *
* IMAC Ing3 2002/2003 - Universit de Marne La Valle  *
* Copyright 2003                                        *
\*******************************************************/
#include "WaterSurface2.h"


/*******************************************************\
*                    constructeur                       *
\*******************************************************/
WaterSurface2::WaterSurface2(GLint newLength, GLint newWidth){
	// mettre les valeurs
	length = newLength;
	width = newWidth;
	
	// crer la grille des z
	for (int i=0; i<length; i++){
		for (int j=0; j<width; j++){
			z[i][j] = 0;	
		}
		glEnd();
	}
	for ( i=0; i<length; i++){
		for (int j=0; j<width; j++){
			nz[i][j] = 0;	
		}
		glEnd();
	}
		
	// crer la grille des v
	for ( i=0; i<length; i++){
		for (int j=0; j<width; j++){
			v[i][j] = 0;	
		}
		glEnd();
	}
	for ( i=0; i<length; i++){
		for (int j=0; j<width; j++){
			nv[i][j] = 0;	
		}
		glEnd();
	}
		
    // dfinir la masse des points 
	mass = 1;
	
	// dfinir la raideur k par dfaut du ressort
	k = 0.1; 

	// perte d'energie
	friction = 0.9995;
   
	// bouger trois points au milieu (pour dmonstration)
	z[20][14] = 5;
	z[20][15] = 5;
	z[20][16] = 5;

	// dfinir le temps coul (utilis dans update)
	time = glutGet(GLUT_ELAPSED_TIME);
}



/*******************************************************\
* Mthode:		setK									*
* Description:  dfinit la raideur des ressorts			*
\*******************************************************/
void WaterSurface2::setK(GLdouble x){
	k = x;
}



/*******************************************************\
* Mthode:		getFriction								*
* Description:  fournit la perte d'energie par friction	*
\*******************************************************/
GLdouble WaterSurface2::getFriction(){
	return friction;
}



/*******************************************************\
* Mthode:		setFriction								*
* Description:  dfinit la perte d'energie par friction	*
\*******************************************************/
void WaterSurface2::setFriction(GLdouble x){
	friction = x;
}



/*******************************************************\
* Mthode:		getMass								*
* Description:  fournit la masse d'un point		*
\*******************************************************/
GLdouble WaterSurface2::getMass(){
	return mass;
}



/*******************************************************\
* Mthode:		setMass								*
* Description:  dfinit la masse   	*
\*******************************************************/
void WaterSurface2::setMass(GLdouble x){
	 mass = x;
}



/*******************************************************\
* Mthode:		getK									*
* Description:  fournit la raideur des ressorts			*
\*******************************************************/
GLdouble WaterSurface2::getK(){
	return k;
}



/*******************************************************\
* Mthode:	reset					*
* Description:  reinitialise la surface			*
\*******************************************************/
void WaterSurface2::reset(){
	// initialiser la grille des z
	for (int i=0; i<length; i++){
		for (int j=0; j<width; j++){
			z[i][j] = 0;	
		}
		glEnd();
	}
	
	// initialiser la grille des v
	for ( i=0; i<length; i++){
		for (int j=0; j<width; j++){
			v[i][j] = 0;	
		}
		glEnd();
	}

	// bouger trois points au milieu (pour dmonstration)
	z[20][14] = 1;
	z[20][15] = 1;
	z[20][16] = 1;
}






/*******************************************************\
* Mthode:		update						*
* Description:  tablit le prochain tat de la surface	*
\*******************************************************/
void WaterSurface2::update(){
	GLdouble x, f;
	GLdouble timestep;

	// dimensionner en fonction du temps coul (en secondes)
	timestep = (glutGet(GLUT_ELAPSED_TIME) - time) / 1000;

	// update de nx et nv
	for (int i=1;i<40-1;i++){
		for (int j=1;j<29;j++){
			// haut
			x = z[i-1][j]-z[i][j];
			f = k * x;

			// bas
			x = z[i+1][j]-z[i][j];
			f = f + k * x;

			// gauche
			x = z[i][j-1]-z[i][j];
			f = f + k * x;

			// droite
			x = z[i][j+1]-z[i][j];
			f = f + k * x;

			// dimensionner f en fonction du temps coul
			f = f * timestep;

			// intgrer...
			v[i][j] = v[i][j]*friction;
			nv[i][j] = mass * f + v[i][j];
			nz[i][j] = z[i][j] + nv[i][j]; 
		}
		glEnd();
	}
	
	// copier nz et nv
	for ( i=0; i<length; i++){
		for (int j=0; j<width; j++){
			v[i][j] = nv[i][j];
			z[i][j] = nz[i][j];				
		}
		glEnd();
	}

	// sauvegarder le temps
	time = glutGet(GLUT_ELAPSED_TIME);
}



/*******************************************************\
* Mthode:		dessiner								*
* Description:  dessine la surface d'eau                *
\*******************************************************/
void WaterSurface2::dessiner(){
	
	glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	glDisable(GL_LIGHTING);
	glColor3f(0.0,0.65,0.8);
	
	glPushMatrix();

	for (int i=1;i<40-2;i++){
		glBegin(GL_QUAD_STRIP);
		for (int j=1;j<29;j++){
			// dessiner les points
			glVertex3d(i,j,z[i][j]);
			glVertex3d(i+1,j,z[i+1][j]);
		}
		glEnd();
	}

	glPopMatrix();

	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
}
