/*******************************************************\
* modlisation d'une surface d'eau                      *
* WaterSurface3.h                                       *
* ------------------------------------------------------*
* Georges AZZI / Georges CAMY                           * 
* Julie POULETTY / Arnaud QUENUM                        *
* IMAC Ing3 2002/2003 - Universit de Marne La Valle  *
* Copyright 2003                                        *
\*******************************************************/
#ifndef WaterSurface3_h
#define WaterSurface3_h
#endif

#ifdef WIN32
#include <windows.h>
#endif

#include <math.h>

#ifndef M_PI
#define M_PI	3.1415926535897932384626433832795
#endif

#include <iostream.h>
#include <fstream.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include <stdio.h>
#include <stdlib.h>


/*******************************************************\
  Classe:	WaterSurface3				
  Description:  the water surface	 
\*******************************************************/
class WaterSurface3{

public :
	WaterSurface3(GLint newLength, GLint newWidth);
    void SetWindSpeed(float speed);
    float GetWindSpeed();
    void SetWindDirection(float direction);
    float GetWindDirection();
    void SetCurrentWave(int w);
    int GetCurrentWave();
    void PrintData();
    void reset();
	void update();
	void dessiner();

private:
  	typedef struct
	{
	   double direction;              // direction 
	   double dirX;                   // direction x part 
	   double dirY;                   // direction y part
	   double lambda;                 // wavelength
	   double k;                      // wave number: k=lambda/(2*Pi)
	   double omega;                  // angular frequency: omega=2*Pi*f
	   double freq;                   // frequency
	   double periode;                // periode of the wave
	   double amplitude;              // amplitude of the wave
	   double phase;                  // phase of the wave
	} WAVE;

	GLdouble z[128][128];		// should be dynamic (but fixed to 128*128)
	GLint length;			// length of the wavefield
	GLint width;			// width of the wavefield
	GLdouble time;			// time
    	GLdouble alpha;        	 	// constant from the Pierson-Moskowitz model
    	GLdouble g;             	// gravitiy
    	GLdouble omega;         	// surface
    	float fPeak;            	// maximum frequency
    	int maxwave;            	// maximum number of calculated waves 
    	float waveDir[ 20 ][ 2 ];	// direction of wave (should be dynamic, 20 = maxwave)
     	int nWaves;			// number of calculated waves
    	WAVE wave[ 20 ];		// the waves (should be dynamic, 20 = maxwave)
    	float windSpeed;        	// windspeed
    	float windDirection;    	// winddirection in radianst
    	int currentWave;        	// actual wave (wave = maxwave is the combined wave)

    	double Gamma (double x);					// some sort of generalized faculty function
    	double MitsuyasuDistribution (double f, double theta);		// angular distribution 
    	double EPM(double f);						// energy from the Pierson.Moskowitz model
    	double EnergyDistribution(double f, double theta);		// combination from angular distribution and pm
    	double Amplitude(double f, double theta, double k);		// amplitude of the wave
};
